#region Using

using System;
using System.IO;
using System.Net;
using System.Xml;
using System.Web;

#endregion

public class XML_RPC : IHttpHandler
{

	/// <summary>
	/// Handles the URL that was sent by the ping request.
	/// </summary>
	private void HandleUrl(Uri url)
	{
		//This is where you write the code for handing the incoming URL.
		//It is the only place you need to change.
		HttpContext.Current.Response.Write(url);
	}

	#region Parse the request

	/// <summary>
	/// Parses the request body and returns the URL.
	/// </summary>
	private static string ParseXml(string xml)
	{
		XmlDocument doc = new XmlDocument();
		doc.LoadXml(xml);

		string methodName = doc.SelectSingleNode("//methodName").InnerText;
		if (methodName == "weblogUpdates.ping")
		{
			XmlNodeList values = doc.SelectNodes("//value");
			if (values.Count == 2)
			{
				return values[1].InnerText;
			}
		}

		throw new NotSupportedException("'" + methodName + "' XML-RPC method not supported");
	}

	/// <summary>
	/// Receives this content from the current HTTP request stream.
	/// </summary>
	public static string ReceiveBody()
	{
		using (StreamReader reader = new StreamReader(HttpContext.Current.Request.InputStream))
		{
			return reader.ReadToEnd();
		}
	}

	#endregion

	#region IHttpHandler members

	/// <summary>
	/// Gets a value indicating whether another request can use the <see cref="T:System.Web.IHttpHandler"></see> instance.
	/// </summary>
	/// <value></value>
	/// <returns>true if the <see cref="T:System.Web.IHttpHandler"></see> instance is reusable; otherwise, false.</returns>
	public bool IsReusable
	{
		get { return false; }
	}

	/// <summary>
	/// Enables processing of HTTP Web requests by a custom HttpHandler that implements the <see cref="T:System.Web.IHttpHandler"></see> interface.
	/// </summary>
	/// <param name="context">An <see cref="T:System.Web.HttpContext"></see> object that provides references to the intrinsic server objects (for example, Request, Response, Session, and Server) used to service HTTP requests.</param>
	public void ProcessRequest(HttpContext context)
	{
		string body = ReceiveBody();

		if (!string.IsNullOrEmpty(body))
		{
			try
			{
				string parsedUrl = ParseXml(body);
				Uri url;

				if (Uri.TryCreate(parsedUrl, UriKind.Absolute, out url))
					HandleUrl(url);
			}
			catch (Exception ex)
			{
				// Put whatever error handling you like here.
				context.Response.Write(ex.Message);
			}
		}
	}

	#endregion

}